==> flutter_slick-3.6.1.tar.gz: lib/components/fields/duration.dart <==
import 'package:flutter/material.dart';
import 'package:flutter_slick/components/fields/text.dart';
import 'package:flutter_slick/utils/parse.dart';

class StandardDurationField extends StatefulWidget {
  final String inputFormat;
  final Duration? initialValue;
  final String? initialStringValue;
  final Function(String value)? onStringChanged;
  final Function(Duration value)? onChanged;
  final FormFieldValidator<String>? validator;
  final Function()? onEditingComplete;
  final Function()? onFocusLost;
  final bool? enabled;
  final bool? autofocus;
  final String? labelText;
  final GestureTapCallback? onTap;
  final Iterable<String>? autofillHints;
  final InputDecoration? decoration;
  final TextStyle? style;

  const StandardDurationField({
    super.key,
    this.inputFormat = 'HH:MM',
    this.onChanged,
    this.initialValue,
    this.initialStringValue,
    this.onStringChanged,
    this.validator,
    this.onEditingComplete,
    this.onFocusLost,
    required this.enabled,
    required this.autofocus,
    this.labelText,
    this.onTap,
    this.autofillHints,
    this.decoration,
    this.style,
  }) : assert(((initialValue != null &&
                initialStringValue == null &&
                onChanged != null &&
                onStringChanged == null) ||
            (initialValue == null &&
                initialStringValue != null &&
                onChanged == null &&
                onStringChanged != null)));

  @override
  State<StandardDurationField> createState() => _StandardDurationFieldState();
}

class _StandardDurationFieldState extends State<StandardDurationField> {
  final _controller = TextEditingController();
  bool _hasAddedColon = false;
  Duration _currentValue = Duration.zero;

  @override
  void initState() {
    if (widget.inputFormat != 'HH:MM') {
      throw ArgumentError(
          'Only HH:MM format is supported. Please update this if you need more formats');
    }
    super.initState();
    _controller.addListener(_formatInput);
  }

  @override
  void dispose() {
    _controller.dispose();
    super.dispose();
  }

  void _formatInput() {
    String currentText = _controller.text;

    final formatted = _processInput(currentText);

    if (formatted != currentText) {
      _controller.value = _controller.value.copyWith(
        text: formatted,
        selection: TextSelection.collapsed(offset: formatted.length),
      );
    }

    var value = parseDuration(formatted);
    if (value != _currentValue) {
      if (widget.onChanged != null) {
        widget.onChanged!(value);
      }
      if (widget.onStringChanged != null) {
        widget.onStringChanged!(formatted);
      }
      _currentValue = value;
    }
  }

  String _processInput(String input) {
    if (input == '') {
      _hasAddedColon = false;
      return input;
    }

    if (input.length > 5) {
      return input.substring(0, 5);
    }

    input = input.replaceAll(RegExp(r"[^\d:]"), "");

    if (!_hasAddedColon && input.length == 2 && !input.contains(':')) {
      input += ':';
      _hasAddedColon = true;
    }

    if (input.contains(':')) {
      var minutes = input.split(':').last;
      if (minutes.length == 1) {
        if (int.parse(minutes) > 5) {
          input = input.substring(0, input.length - 1);
        }
      }
      if (minutes.length == 3) {
        input = input.substring(0, input.length - 1);
      }
    }

    input = input.replaceAll('::', ':');

    return input;
  }

  @override
  Widget build(BuildContext context) {
    return StandardTextField(
      controller: _controller,
      keyboardType: TextInputType.number,
      validator: widget.validator,
      onEditingComplete: widget.onEditingComplete,
      onFocusLost: widget.onFocusLost,
      labelText: widget.labelText ?? 'HH:MM',
      enabled: widget.enabled ?? true,
      autofocus: widget.autofocus ?? false,
      onChanged: (String? v) {},
      onTap: widget.onTap,
      autofillHints: widget.autofillHints,
      decoration: widget.decoration,
      style: widget.style,
    );
  }
}

==> flutter_slick-4.0.1.tar.gz: lib/components/fields/duration.dart <==
import 'package:flutter/material.dart';
import 'package:flutter_slick/components/fields/text.dart';
import 'package:flutter_slick/utils/parse.dart';

class StandardDurationField extends StatefulWidget {
  final String inputFormat;
  final Duration? initialValue;
  final String? initialStringValue;
  final Function(String value)? onStringChanged;
  final Function(Duration value)? onChanged;
  final FormFieldValidator<String>? validator;
  final Function()? onEditingComplete;
  final Function()? onFocusLost;
  final bool enabled;
  final bool autofocus;
  final String? labelText;
  final GestureTapCallback? onTap;
  final Iterable<String>? autofillHints;
  final InputDecoration? decoration;
  final TextStyle? style;

  const StandardDurationField({
    super.key,
    this.inputFormat = 'HH:MM',
    this.onChanged,
    this.initialValue,
    this.initialStringValue,
    this.onStringChanged,
    this.validator,
    this.onEditingComplete,
    this.onFocusLost,
    this.enabled = true,
    this.autofocus = false,
    this.labelText,
    this.onTap,
    this.autofillHints,
    this.decoration,
    this.style,
  }) : assert(((initialValue != null &&
                initialStringValue == null &&
                onChanged != null &&
                onStringChanged == null) ||
            (initialValue == null &&
                initialStringValue != null &&
                onChanged == null &&
                onStringChanged != null)));

  @override
  State<StandardDurationField> createState() => _StandardDurationFieldState();
}

class _StandardDurationFieldState extends State<StandardDurationField> {
  final _controller = TextEditingController();
  bool _hasAddedColon = false;
  Duration _currentValue = Duration.zero;

  @override
  void initState() {
    if (widget.inputFormat != 'HH:MM') {
      throw ArgumentError(
          'Only HH:MM format is supported. Please update this if you need more formats');
    }
    super.initState();
    _controller.addListener(_formatInput);
  }

  @override
  void dispose() {
    _controller.dispose();
    super.dispose();
  }

  void _formatInput() {
    String currentText = _controller.text;

    final formatted = _processInput(currentText);

    if (formatted != currentText) {
      _controller.value = _controller.value.copyWith(
        text: formatted,
        selection: TextSelection.collapsed(offset: formatted.length),
      );
    }

    var value = parseDuration(formatted);
    if (value != _currentValue) {
      if (widget.onChanged != null) {
        widget.onChanged!(value);
      }
      if (widget.onStringChanged != null) {
        widget.onStringChanged!(formatted);
      }
      _currentValue = value;
    }
  }

  String _processInput(String input) {
    if (input == '') {
      _hasAddedColon = false;
      return input;
    }

    if (input.length > 5) {
      return input.substring(0, 5);
    }

    input = input.replaceAll(RegExp(r"[^\d:]"), "");

    if (!_hasAddedColon && input.length == 2 && !input.contains(':')) {
      input += ':';
      _hasAddedColon = true;
    }

    if (input.contains(':')) {
      var minutes = input.split(':').last;
      if (minutes.length == 1) {
        if (int.parse(minutes) > 5) {
          input = input.substring(0, input.length - 1);
        }
      }
      if (minutes.length == 3) {
        input = input.substring(0, input.length - 1);
      }
    }

    input = input.replaceAll('::', ':');

    return input;
  }

  @override
  Widget build(BuildContext context) {
    return StandardTextField(
      controller: _controller,
      keyboardType: TextInputType.number,
      validator: widget.validator,
      onEditingComplete: widget.onEditingComplete,
      onFocusLost: widget.onFocusLost,
      labelText: widget.labelText ?? 'HH:MM',
      enabled: widget.enabled,
      autofocus: widget.autofocus,
      onChanged: (String? v) {},
      onTap: widget.onTap,
      autofillHints: widget.autofillHints,
      decoration: widget.decoration,
      style: widget.style,
    );
  }
}
